==> form-builder-vue-0.0.44.tgz: package/src/components/widgets/FormWidget.ts <==
import { deepAssign } from '@codeperate/utils';
import { ComputedRef, Ref, computed, inject, onBeforeMount, onUnmounted, ref, toRaw } from 'vue';
import { ExportValueOptions, FormSchema, ValidatedMeta } from '../FormBuilder.interface';
import FormBuilder from '../FormBuilder.vue';
import { useConfig } from '../config';

export interface FormWidgetProps {
    path: string;
}
export interface IFormWidget<C = any, V = any> {
    isValidated: Ref<boolean>;
    getSchema: () => FormSchema;
    value: Ref<V>;
    getPath: () => string;
    setValue: (value: any) => void;
    isRequired: () => boolean;
    isValid: () => boolean | undefined;
    isHidden: () => boolean;
    register: (exposed: Partial<IFormWidget> & Record<any, any>) => IFormWidget;
    form: InstanceType<typeof FormBuilder>;
    validator: () => void;
    validate: () => ValidatedMeta;
    undoValidate: () => void;
    validatedMeta: Ref<ValidatedMeta>;
    view: Ref<boolean>;
    schema: FormSchema;
    config: ComputedRef<C>;
    onExportValue: (wrappedValue: { value: any }, options: ExportValueOptions) => Promise<any>;
    onSaveHistory: (value: any) => Promise<void>;
    onLoadHistory: (value: any) => Promise<void>;
    el: Ref<HTMLElement | undefined>;
}
export function useFormWidget<C = any, V = any, N extends string = any>(name: N, props: FormWidgetProps): Omit<IFormWidget<C, V>, 'el'> {
    const isValidated = ref<boolean>(false);
    const validatedMeta = ref<ValidatedMeta>({ validity: true, path: props.path });
    const form = inject<InstanceType<typeof FormBuilder>>('formBuilder')!;
    let widget: IFormWidget;
    const isHidden = () => {
        let hidden = schema.hidden;
        if (hidden) return typeof hidden == 'function' ? hidden(widget) : hidden;
        return false;
    };
    const getSchema = () => {
        return form!.getSchema(props.path);
    };
    const getValue = () => {
        return form!.getValue(getPath());
    };

    const getPath = () => {
        return getSchema().targetPath ?? props.path;
    };

    const setValue = (value: V) => {
        form!.setValue(getPath(), value);
    };
    const isRequired = () => {
        const required = getSchema().required;
        if (required == null) return false;
        return typeof required == 'function' ? required(widget) : required;
    };
    const isValid = () => {
        if (isValidated.value == true) return validatedMeta.value.validity;
    };
    const value = computed(() => {
        return getValue();
    });
    const schema = getSchema();
    const config = computed(() => {
        return deepAssign(useConfig(name), schema?.config ?? {});
    });
    const register = _exposed => {
        widget = { ...exposed, ...(_exposed ?? {}) };
        onBeforeMount(() => form!.regWidget(props.path, widget));
        onUnmounted(() => form!.unRegWidget(props.path, widget));
        return widget;
    };
    const validator = () => {
        validatedMeta.value.validity = true;
    };
    const view = computed(() => form.getView() ?? schema.view);
    const validate = () => {
        let result = toRaw(validatedMeta.value);
        const schema = getSchema();
        let { validate } = schema;
        validate = typeof validate == 'function' ? validate(widget) : validate;
        if (!view.value && (validate ?? true)) {
            if (schema.validateFn) schema.validateFn(value.value, widget);
            else widget.validator();
            isValidated.value = true;
        }
        return result;
    };
    const undoValidate = () => {
        isValidated.value = false;
        validatedMeta.value.validity = false;
    };
    if (getSchema().defaultValue !== undefined && value.value == undefined) setValue(getSchema().defaultValue as V);
    onBeforeMount(() => {
        if (widget == undefined) throw new Error('You must call register() function');
    });
    // onRenderTriggered(({ key, target, type }) => {
    //     console.log(name);
    //     console.log(`Render triggered on component`, { key, target, type });
    // });
    const onExportValue = async () => {};
    const onSaveHistory = async () => {};
    const onLoadHistory = async () => {};
    const exposed = {
        isValidated,
        form,
        getSchema,
        isRequired,
        value,
        getPath,
        setValue,
        getValue,
        register,
        validator,
        validate,
        undoValidate,
        validatedMeta,
        view,
        schema,
        config,
        isHidden,
        isValid,
        onExportValue,
        onSaveHistory,
        onLoadHistory,
    };
    return exposed;
}

==> form-builder-vue-0.1.0.tgz: package/src/components/widgets/FormWidget.ts <==
import { deepAssign } from '@codeperate/utils';
import { Ref, computed, inject, onBeforeMount, onUnmounted, ref, toRaw } from 'vue';
import { ExportValueOptions, FormSchema, ValidatedMeta } from '../FormBuilder.interface';
import FormBuilder from '../FormBuilder.vue';
import { useConfig } from '../config';

export interface FormWidgetProps {
    path: string;
}
export interface IFormWidget<V = any> {
    isValidated: Ref<boolean>;
    getSchema: () => FormSchema;
    value: Ref<V>;
    getPath: () => string;
    setValue: (value: any) => void;
    isRequired: () => boolean;
    isValid: () => boolean | undefined;
    isHidden: () => boolean;
    register: (exposed: Partial<IFormWidget> & Record<any, any>) => IFormWidget;
    form: InstanceType<typeof FormBuilder>;
    validator: () => void;
    validate: () => ValidatedMeta;
    undoValidate: () => void;
    validatedMeta: Ref<ValidatedMeta>;
    view: Ref<boolean>;
    schema: FormSchema;
    onExportValue: (wrappedValue: { value: any }, options: ExportValueOptions) => Promise<any>;
    onSaveHistory: (value: any) => Promise<void>;
    onLoadHistory: (value: any) => Promise<void>;
    el: Ref<HTMLElement | undefined>;
}
export function useFormWidget<V = any, N extends string = any>(name: N, props: FormWidgetProps): Omit<IFormWidget<V>, 'el'> {
    const isValidated = ref<boolean>(false);
    const validatedMeta = ref<ValidatedMeta>({ validity: true, path: props.path });
    const form = inject<InstanceType<typeof FormBuilder>>('formBuilder')!;
    let widget: IFormWidget;
    const isHidden = () => {
        let hidden = schema.hidden;
        if (hidden) return typeof hidden == 'function' ? hidden(widget) : hidden;
        return false;
    };
    const getSchema = () => {
        return form!.getSchema(props.path);
    };
    const getValue = () => {
        return form!.getValue(getPath());
    };

    const getPath = () => {
        return getSchema().targetPath ?? props.path;
    };

    const setValue = (value: V) => {
        form!.setValue(getPath(), value);
    };
    const isRequired = () => {
        const required = getSchema().required;
        if (required == null) return false;
        return typeof required == 'function' ? required(widget) : required;
    };
    const isValid = () => {
        if (isValidated.value == true) return validatedMeta.value.validity;
    };
    const value = computed(() => {
        return getValue();
    });
    const schema = getSchema();
    const config = computed(() => {
        return deepAssign(useConfig(name), schema?.config ?? {});
    });
    const register = _exposed => {
        widget = { ...exposed, ...(_exposed ?? {}) };
        onBeforeMount(() => form!.regWidget(props.path, widget));
        onUnmounted(() => form!.unRegWidget(props.path, widget));
        return widget;
    };
    const validator = () => {
        validatedMeta.value.validity = true;
    };
    const view = computed(() => form.getView() ?? schema.view);
    const validate = () => {
        let result = toRaw(validatedMeta.value);
        const schema = getSchema();
        let { validate } = schema;
        validate = typeof validate == 'function' ? validate(widget) : validate;
        if (!view.value && (validate ?? true)) {
            if (schema.validateFn) schema.validateFn(value.value, widget);
            else widget.validator();
            isValidated.value = true;
        }
        return result;
    };
    const undoValidate = () => {
        isValidated.value = false;
        validatedMeta.value.validity = false;
    };
    if (getSchema().defaultValue !== undefined && value.value == undefined) setValue(getSchema().defaultValue as V);
    onBeforeMount(() => {
        if (widget == undefined) throw new Error('You must call register() function');
    });
    // onRenderTriggered(({ key, target, type }) => {
    //     console.log(name);
    //     console.log(`Render triggered on component`, { key, target, type });
    // });
    const onExportValue = async () => {};
    const onSaveHistory = async () => {};
    const onLoadHistory = async () => {};
    const exposed = {
        isValidated,
        form,
        getSchema,
        isRequired,
        value,
        getPath,
        setValue,
        getValue,
        register,
        validator,
        validate,
        undoValidate,
        validatedMeta,
        view,
        schema,
        config,
        isHidden,
        isValid,
        onExportValue,
        onSaveHistory,
        onLoadHistory,
    };
    return exposed;
}
